"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createContext = exports.Context = void 0;
const objectToString = (obj) => {
    try {
        return JSON.stringify(obj, null, 2);
    }
    catch (_a) {
        return obj.toString();
    }
};
class Context extends Map {
    constructor() {
        super();
    }
    delete(key, callback = () => { }) {
        const value = this.get(key);
        const ret = super.delete(key);
        callback(value);
        return ret;
    }
    get [Symbol.toStringTag]() {
        const arr = [];
        for (const [key, value] of this) {
            arr.push(`[${key.toString()}]: <${typeof value}>${typeof value === 'object' ? objectToString(value) : value.toString()}`);
        }
        return `{ ${arr.join(', ')} }`;
    }
}
exports.Context = Context;
exports.createContext = () => {
    const context = new Context();
    return new Proxy(context, {
        get: (target, prop, receiver) => {
            if (prop in target) {
                return target[prop].bind(target);
            }
            return target.get(prop);
        },
        set: (target, prop, value) => {
            target.set(prop, value);
            return !!target.get(prop);
        },
        has: (target, key) => {
            return target.has(key);
        },
        deleteProperty: (target, key) => {
            if (target.has(key)) {
                target.delete(key);
                return !target.has(key);
            }
            return false;
        },
    });
};
