"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AddCorrelationId = void 0;
const common_1 = require("@nestjs/common");
const lodash_1 = require("lodash");
const src_1 = require("../../../../nestjs-context/src");
const constants_1 = require("../../constants");
exports.AddCorrelationId = (correlationIdPath) => (Target) => {
    if (!correlationIdPath.length) {
        throw new common_1.InternalServerErrorException('Correlation id path must have at least 1 character');
    }
    const correlationIdPathParts = correlationIdPath.split('.');
    const accessorPropertyName = correlationIdPathParts.shift();
    const dataPropertyName = `__add-correlation-id-${accessorPropertyName}`;
    const newClass = class extends Target {
        constructor(...args) {
            super(...args);
            const descriptor = lodash_1.cloneDeep(this[accessorPropertyName]);
            Object.defineProperty(this, dataPropertyName, {
                enumerable: false,
                value: descriptor,
            });
            Object.defineProperty(this, accessorPropertyName, {
                enumerable: true,
                get() {
                    if (correlationIdPathParts.length === 0) {
                        return src_1.defaultContext.get(constants_1.CONTEXT_CORRELATION_ID);
                    }
                    return lodash_1.set(this[dataPropertyName], correlationIdPathParts, src_1.defaultContext.get(constants_1.CONTEXT_CORRELATION_ID));
                },
                set(value) {
                    this[dataPropertyName] = value;
                },
            });
        }
    };
    Object.defineProperty(newClass, 'name', {
        value: Target.name,
    });
    return newClass;
};
