"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PinoContextLogger = void 0;
const common_1 = require("@nestjs/common");
const createPinoLogger = require("pino");
const lodash_1 = require("lodash");
const nestjs_context_1 = require("../../nestjs-context");
const pino_context_config_1 = require("./pino-context.config");
const enums_1 = require("./enums");
const type_guards_1 = require("./type-guards");
const constants_1 = require("./constants");
let PinoContextLogger = class PinoContextLogger {
    constructor(config = new pino_context_config_1.PinoContextConfig(), contextStorage = nestjs_context_1.createContext(), logger = createPinoLogger(config.loggerOptions)) {
        this.config = config;
        this.contextStorage = contextStorage;
        this.logger = logger;
    }
    setContext(context) {
        this.logger = this.logger.child({ context: context });
        this.context = context;
    }
    hasContext() {
        return !!this.context;
    }
    buildFieldsForConfigHelper(configKey, configName, getFieldValueCallback) {
        return lodash_1.fromPairs(this.config[configKey][configName].map(fieldConfig => {
            let label;
            let path = [];
            if (typeof fieldConfig === 'string') {
                label = fieldConfig;
            }
            else {
                let pathConfig;
                ({ label, path: pathConfig = '' } = fieldConfig);
                path =
                    typeof pathConfig === 'string'
                        ? pathConfig.split('.').filter(s => s.length)
                        : pathConfig;
            }
            path = path.length ? path : [label];
            return [label, getFieldValueCallback(path, fieldConfig)];
        }));
    }
    buildFieldsForContextConfig(configKey) {
        return this.buildFieldsForConfigHelper(configKey, 'context', (path) => {
            return lodash_1.get(this.contextStorage, path);
        });
    }
    buildFieldsForRequestConfig(configKey) {
        return this.buildFieldsForConfigHelper(configKey, 'request', (path) => lodash_1.get(this.contextStorage, [
            constants_1.CONTEXT_PINO_LOGGER_REQUEST,
            configKey,
            ...path,
        ]));
    }
    buildFieldsForEnvConfig(configKey) {
        return this.buildFieldsForConfigHelper(configKey, 'env', (path, fieldConfig) => {
            const value = fieldConfig
                .value;
            return typeof value !== 'undefined' ? value : lodash_1.get(process.env, path);
        });
    }
    buildFields(configKey) {
        if (!type_guards_1.hasConfiguration(this.config, [configKey])) {
            return {};
        }
        return lodash_1.pickBy(Object.assign(Object.assign(Object.assign({}, this.buildFieldsForContextConfig(configKey)), this.buildFieldsForRequestConfig(configKey)), this.buildFieldsForEnvConfig(configKey)), (value) => !lodash_1.isEmpty(value));
    }
    setLabel(label, value) {
        const labelDef = {
            label,
            value,
        };
        if (!this.config.labels.env.includes(labelDef)) {
            this.config.labels.env.push(labelDef);
        }
        return this;
    }
    getMergingObject({ context, trace, }) {
        return lodash_1.pickBy(Object.assign(Object.assign({}, this.buildFields(enums_1.LogFieldsConfigKey.FIELDS)), { [this.config.getFieldNameContext()]: context, [this.config.getFieldNameTrace()]: trace, [this.config.getFieldNameLabels()]: this.buildFields(enums_1.LogFieldsConfigKey.LABELS) }), (value) => !lodash_1.isEmpty(value));
    }
    verbose(message, context, ...args) {
        return this.logger.trace(this.getMergingObject({ context }), message, ...args);
    }
    debug(message, context, ...args) {
        return this.logger.debug(this.getMergingObject({ context }), message, ...args);
    }
    log(message, context, ...args) {
        return this.logger.info(this.getMergingObject({ context }), message, ...args);
    }
    warn(message, context, ...args) {
        return this.logger.warn(this.getMergingObject({ context }), message, ...args);
    }
    error(message, trace, context, ...args) {
        if (message instanceof Error) {
            message = message.message;
            trace = trace || message.stack;
        }
        return this.logger.error(this.getMergingObject({ trace, context }), message, ...args);
    }
};
PinoContextLogger = __decorate([
    common_1.Injectable({ scope: common_1.Scope.TRANSIENT }),
    __param(0, common_1.Optional()),
    __param(1, common_1.Optional()),
    __param(2, common_1.Optional()),
    __metadata("design:paramtypes", [pino_context_config_1.PinoContextConfig,
        nestjs_context_1.Context, Object])
], PinoContextLogger);
exports.PinoContextLogger = PinoContextLogger;
