"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PinoContextRequestInterceptor = void 0;
const common_1 = require("@nestjs/common");
const operators_1 = require("rxjs/operators");
const nestjs_ps_tools_1 = require("../../nestjs-ps-tools");
const nestjs_context_1 = require("../../nestjs-context");
const constants_1 = require("./constants");
const pino_context_config_1 = require("./pino-context.config");
const type_guards_1 = require("./type-guards");
const lodash_1 = require("lodash");
const enums_1 = require("./enums");
let PinoContextRequestInterceptor = class PinoContextRequestInterceptor {
    constructor(context, config) {
        this.context = context;
        this.config = config;
    }
    intercept(context, next) {
        const httpContext = context.switchToHttp();
        const request = httpContext.getRequest();
        this.addContextRequestValues(request);
        return next.handle().pipe(operators_1.tap(() => this.context.set(constants_1.CONTEXT_PINO_LOGGER_REQUEST, {})));
    }
    addContextRequestValues(request) {
        for (const field of Object.values(enums_1.LogFieldsConfigKey)) {
            if (type_guards_1.hasConfiguration(this.config, [field, 'request'])) {
                this.config[field].request.forEach(requestFieldDef => {
                    const { label, pick, filter = (req) => true } = requestFieldDef;
                    this.addContextRequestValue(request, field, label, pick, filter);
                });
            }
        }
    }
    addContextRequestValue(request, configFieldsKey, label, pick, filter) {
        if (!filter(request)) {
            return;
        }
        Object.values(nestjs_ps_tools_1.pickFromRequestTool(request, pick)).forEach(value => {
            const reqLabels = this.context.get(constants_1.CONTEXT_PINO_LOGGER_REQUEST) || {};
            lodash_1.set(reqLabels, [configFieldsKey, label], value);
            this.context.set(constants_1.CONTEXT_PINO_LOGGER_REQUEST, reqLabels);
        });
    }
};
PinoContextRequestInterceptor = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [nestjs_context_1.Context,
        pino_context_config_1.PinoContextConfig])
], PinoContextRequestInterceptor);
exports.PinoContextRequestInterceptor = PinoContextRequestInterceptor;
