"use strict";
var _a, _b;
Object.defineProperty(exports, "__esModule", { value: true });
exports.PinoContextConfig = void 0;
const lodash_1 = require("lodash");
const configs = require("./config");
const type_guards_1 = require("./type-guards");
const enums_1 = require("./enums");
const defaultLogFieldNames = {
    context: 'context',
    labels: 'labels',
    trace: 'trace',
};
class PinoContextConfig {
    constructor(config = {}) {
        this[_a] = {
            env: [],
            request: [],
            context: [
                'logging.googleapis.com/trace',
            ],
        };
        this[_b] = {
            env: [],
            request: [],
            context: ['x-correlation-id'],
        };
        this.loggerOptions = {};
        this.logFieldNames = defaultLogFieldNames;
        let configToMerge = {};
        if (type_guards_1.isPredefinedLogger(config)) {
            configToMerge = configs[config];
        }
        else if (type_guards_1.isCustomLogger(config)) {
            const defaultValues = config.base
                ? configs[config.base]
                : {};
            configToMerge = lodash_1.merge(defaultValues, config);
        }
        lodash_1.merge(this, configToMerge);
    }
    getFieldNameContext() {
        return this.logFieldNames.context;
    }
    getFieldNameLabels() {
        return this.logFieldNames.labels;
    }
    getFieldNameTrace() {
        return this.logFieldNames.trace;
    }
}
exports.PinoContextConfig = PinoContextConfig;
_a = enums_1.LogFieldsConfigKey.FIELDS, _b = enums_1.LogFieldsConfigKey.LABELS;
