"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLoggerTool = void 0;
const path = require("path");
const pino_context_logger_service_1 = require("../pino-context-logger.service");
const pino_context_config_1 = require("../pino-context.config");
const nestjs_context_1 = require("../../../nestjs-context");
const type_guards_1 = require("../type-guards");
const nestjs_context_2 = require("../../../nestjs-context");
exports.createLoggerTool = (configOrApp = {}, contextName = process.argv[1]
    ? path.basename(process.argv[1], path.extname(process.argv[1]))
    : path.basename(__filename)) => {
    let logger;
    try {
        let config, context;
        if (type_guards_1.isNestApplication(configOrApp)) {
            config = configOrApp.get(pino_context_config_1.PinoContextConfig);
            context = configOrApp.get(nestjs_context_1.Context);
        }
        else {
            config = new pino_context_config_1.PinoContextConfig(configOrApp);
            context = nestjs_context_2.defaultContext;
        }
        logger = new pino_context_logger_service_1.PinoContextLogger(config, context);
        logger.debug(`Created ${pino_context_logger_service_1.PinoContextLogger.name} with context ${contextName} and config ${JSON.stringify(config)}`);
    }
    catch (e) {
        logger = new pino_context_logger_service_1.PinoContextLogger();
        logger.warn(`Could not create ${pino_context_logger_service_1.PinoContextLogger.name} into the given context. Reason: ${e.toString()}`);
    }
    logger.setContext(contextName);
    return logger;
};
